package compiler

import "git.urbach.dev/cli/q/src/core"

// Statistics contains counters for all functions in the build.
type Statistics struct {
	InstructionCount      int
	DataCount             int
	LabelCount            int
	NumberCount           int
	RegisterCount         int
	RegisterLabelCount    int
	RegisterNumberCount   int
	RegisterRegisterCount int
	MemoryCount           int
	MemoryLabelCount      int
	MemoryNumberCount     int
	MemoryRegisterCount   int
}

// Count adds statistics for this function.
func (s *Statistics) Count(function *core.Function) {
	s.InstructionCount += len(function.Assembler.Instructions)
	s.DataCount += len(function.Assembler.Data)
	s.LabelCount += len(function.Assembler.Param.Label)
	s.NumberCount += len(function.Assembler.Param.Number)
	s.RegisterCount += len(function.Assembler.Param.Register)
	s.RegisterLabelCount += len(function.Assembler.Param.RegisterLabel)
	s.RegisterNumberCount += len(function.Assembler.Param.RegisterNumber)
	s.RegisterRegisterCount += len(function.Assembler.Param.RegisterRegister)
	s.MemoryCount += len(function.Assembler.Param.Memory)
	s.MemoryLabelCount += len(function.Assembler.Param.MemoryLabel)
	s.MemoryNumberCount += len(function.Assembler.Param.MemoryNumber)
	s.MemoryRegisterCount += len(function.Assembler.Param.MemoryRegister)
}
